// --------------------------------------------------------------------------
// File part of qMini, a minimal 3D app to extend.
//
// Copyright(C) 2009-2013
// Tamy Boubekeur
//                                                                            
// All rights reserved.                                                       
//                                                                            
// This program is free software; you can redistribute it and/or modify       
// it under the terms of the GNU General Public License as published by       
// the Free Software Foundation; either version 2 of the License, or          
// (at your option) any later version.                                        
//                                                                            
// This program is distributed in the hope that it will be useful,            
// but WITHOUT ANY WARRANTY; without even the implied warranty of             
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              
// GNU General Public License (http://www.gnu.org/licenses/gpl.txt)           
// for more details.                                                          
// --------------------------------------------------------------------------

#pragma once
#define GLEW_STATIC 1
#include <GL/glew.h>
#include <QGLViewer/qglviewer.h>
#include <QString>
#include <QApplication>
#include <QCoreApplication>
#include <QAction>
#include <QMainWindow>
#include <QFileDialog>
#include <QMenu>
#include <QMenuBar>
#include <QDir>
#include <string>
#include "Shader.h"
#include "Mesh.h"

class GLViewer : public QGLViewer  {
    Q_OBJECT
public:
    void open (const QString & filename);
protected :
    void init();
    void draw ();
    QString helpString() const;
    void keyPressEvent (QKeyEvent * e);
private:
    void initLights ();
    Shader * phongShader;
    Mesh mesh;
};

class GLApp : public QMainWindow {
    Q_OBJECT
public:
    GLApp (const QString & appName, int argc, char ** argv) : QMainWindow () {
        QAction * fileOpenAction = new QAction ("Open", this);
        fileOpenAction->setShortcut (tr ("Ctrl+O"));
        connect (fileOpenAction, SIGNAL (activated()), this, SLOT (open()));
        QAction * fileQuitAction = new QAction ("Quit", this);
        fileQuitAction->setShortcut (tr ("Ctrl+Q"));
        connect (fileQuitAction, SIGNAL (activated()) , qApp, SLOT (closeAllWindows()));
        QActionGroup * fileActionGroup = new QActionGroup (this);
        fileActionGroup->addAction (fileOpenAction);
        fileActionGroup->addAction (fileQuitAction);
        QMenu * fileMenu = menuBar ()->addMenu (tr ("File"));;
        fileMenu->addActions (fileActionGroup->actions ());
        _viewer = new GLViewer;
        setCentralWidget (_viewer);
        setMinimumSize (200, 200);
        setWindowTitle (appName);
        showMaximized ();
        init (argc, argv);
        show ();
    }
    inline GLViewer * viewer () { return _viewer; }
    virtual void init (int argc, char ** argv) {
        if (argc > 1) 
            open (QString ((argv[1][0] == '/'
                            ? std::string (argv [1])
                            : std::string (qPrintable (QDir::currentPath ())) + "/" + std::string (argv [1])).c_str ()));
    }
public slots:
    void open (const QString & filename) { _viewer->open (filename); }
    void open () { open (QFileDialog::getOpenFileName ((QWidget*)this, "Choose the mesh to load", QDir::currentPath (), "OFF (*.off)")); }
private:
    GLViewer * _viewer;
};


// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
